//*----------------------------------------------------------------------------
//*         ATMEL Microcontroller Software Support  -  ROUSSET  -
//*----------------------------------------------------------------------------
//* The software is delivered "AS IS" without warranty or condition of any
//* kind, either express, implied or statutory. This includes without
//* limitation any warranty or condition with respect to merchantability or
//* fitness for any particular purpose, or against the infringements of
//* intellectual property rights of others.
//*----------------------------------------------------------------------------
//* File Name           : Cstartup_SAM7.c
//* Object              : Low level initializations written in C for GCC Tools
//* Creation            : 12/Jun/04
//* 1.2   28/Feb/05 JPP : LIB change AT91C_WDTC_WDDIS & PLL
//* 1.3   21/Mar/05 JPP : Change PLL Wait time
//*----------------------------------------------------------------------------


#define EXT_OC          18432000	// Exetrnal oscillator MAINCK
#define MX_PLL_MUL		26			// PLL multiplier
#define MX_PLL_DIV		5			// PLL divider
#define MX_CLK_DIV		1			// Selected clock source divider (0=1:1, 1=1:2, 2=1:4)

// Flowcode clock speed setting should be EXT_OC * MX_PLL_MUL / MX_PLL_DIV / MX_CLK_DIV (ratio)

// Include the board file description
#include "Board.h"

// The following functions must be write in ARM mode this function called directly
// by exception vector
extern void AT91F_Spurious_handler(void);
extern void AT91F_Default_IRQ_handler(void);
extern void AT91F_Default_FIQ_handler(void);


//*----------------------------------------------------------------------------
//* \fn    AT91F_LowLevelInit
//* \brief This function performs very low level HW initialization
//*        this function can be use a Stack, depending the compilation
//*        optimization mode
//*----------------------------------------------------------------------------
void AT91F_LowLevelInit( void)
{
	// This setup has a 18.432MHz crystal, so MAINCK runs at 18.432MHz
	// Slow clock runs at ~32KHz

	int            i;
	AT91PS_PMC     pPMC = AT91C_BASE_PMC;

	// Setup the flash wait state
	AT91C_BASE_MC->MC_FMR = ((AT91C_MC_FMCN)&(48 <<16)) | AT91C_MC_FWS_1FWS ;

    // Disable the watchdog timer
    AT91C_BASE_WDTC->WDTC_WDMR= AT91C_WDTC_WDDIS;

	//* Set MCK at 47 923 200
    // 1 Enabling the Main Oscillator (MAINCK):
        // Slow clock : SCK = 1/32768 = 0.0003052 sec
    	// Start up time = 8 * 6 / SCK = 56 * 30.51 = 1,46484375 ms
		// Enable the main oscillator, and wait for
       pPMC->PMC_MOR = ( (AT91C_CKGR_OSCOUNT) & (0x06 <<8)) | AT91C_CKGR_MOSCEN ;
        while(!(pPMC->PMC_SR & AT91C_PMC_MOSCS));
	// 2 Checking the Main Oscillator Frequency (Optional)
	// 3 Setting PLL and divider:
		// - div by 5 Fin = 3,6864 =(18,432 / 5)
		// - Mul 25+1: Fout =	95,8464 =(3,6864 *26)
		// for 96 MHz the erroe is 0.16%
		// Field out NOT USED = 0
		// PLLCOUNT pll startup time estimate at : 0.844 ms
		// PLLCOUNT 28 = 0.000844 /(1/32768)
       pPMC->PMC_PLLR = ((AT91C_CKGR_DIV & MX_PLL_DIV) |
                         (AT91C_CKGR_PLLCOUNT & (28<<8)) |
                         (AT91C_CKGR_MUL & ((MX_PLL_MUL - 1)<<16)));

        // Wait the startup time
        while(!(pPMC->PMC_SR & AT91C_PMC_LOCK));
        while(!(pPMC->PMC_SR & AT91C_PMC_MCKRDY));
 	// 4. Selection of Master Clock and Processor Clock

 	// select the PLL clock divided by 2
	//  We want to divide by two
 	    pPMC->PMC_MCKR = (MX_CLK_DIV<<2);
 	    while(!(pPMC->PMC_SR & AT91C_PMC_MCKRDY));

	// And we want to ues the PLL output
 	    pPMC->PMC_MCKR |= AT91C_PMC_CSS_PLL_CLK  ;
 	    while(!(pPMC->PMC_SR & AT91C_PMC_MCKRDY));

	// MCK running at 47 932 200 Hz = ( ( MAINCK / 5 ) * 26 ) / 2

	// Set up the default interrupts handler vectors
	AT91C_BASE_AIC->AIC_SVR[0] = (int) AT91F_Default_FIQ_handler ;
	for (i=1;i < 31; i++)
	{
	    AT91C_BASE_AIC->AIC_SVR[i] = (int) AT91F_Default_IRQ_handler ;
	}
	AT91C_BASE_AIC->AIC_SPU  = (int) AT91F_Spurious_handler ;


	// Enable the PIO controller clock
	AT91F_PMC_EnablePeriphClock ( AT91C_BASE_PMC, 1 << AT91C_ID_PIOA ) ;

	// Set all pins to be controlled by the PIO controller
	AT91F_PIO_Enable( AT91C_BASE_PIOA, 0xffffffff );

	// Enable the reset button
	AT91C_BASE_RSTC->RSTC_RMR = 0x01 | (0x4<<8) | (unsigned int)(0xA5<<24);

	// *** Bootloader selection code ***

	/* Set Ppin  */
	AT91F_PIO_CfgInput( AT91C_BASE_PIOA, 1<<13 | 1<<16 );

	/* Disable pullups */
	AT91F_PIO_DisablePullups( AT91C_BASE_PIOA, 1<<13 | 1<<16 );

	// This delay lets the inputs settle down
	for ( i = 0 ; i < 32000 ; i++ )
	{
	}

	// Is USB connected?
	if ( AT91F_PIO_IsInputSet( AT91C_BASE_PIOA, 1<<13 ) )
	{
		// Is the prog button being pressed?
		if ( !(AT91F_PIO_GetInput(AT91C_BASE_PIOA) & 1<<16 ) )
		{
			// Jump to the bootloader
			asm ("	ldr r15, .bootloaderaddr \n \
			b 0x119000 \n \
			.bootloaderaddr: \n \
			.word 0x119000" );
		}
	}
}

